@@ -16,6 +16,7 @@ limitations under the License.
 
 from cvxpy import settings as s
 from cvxpy import error
+from cvxpy.expressions import cvxtypes
 from cvxpy.problems.objective import Minimize, Maximize
 from cvxpy.reductions.chain import Chain
 from cvxpy.reductions.dgp2dcp.dgp2dcp import Dgp2Dcp
@@ -47,6 +48,39 @@ SolveResult = namedtuple(
     ['opt_value', 'status', 'primal_values', 'dual_values'])
 
 
+_COL_WIDTH = 79
+_HEADER = (
+    '='*_COL_WIDTH +
+    '\n' +
+    ('CVXPY').center(_COL_WIDTH) +
+    '\n' +
+    (f'v{cvxtypes.version()}').center(_COL_WIDTH) +
+    '\n' +
+    '='*_COL_WIDTH
+)
+_COMPILATION_STR = (
+    '-'*_COL_WIDTH +
+    '\n' +
+    ('Compilation').center(_COL_WIDTH) +
+    '\n' +
+    '-'*_COL_WIDTH
+)
+_NUM_SOLVER_STR = (
+    '-'*_COL_WIDTH +
+    '\n' +
+    ('Numerical solver').center(_COL_WIDTH) +
+    '\n' +
+    '-'*_COL_WIDTH
+)
+_FOOTER = (
+    '-'*_COL_WIDTH +
+    '\n' +
+    ('Summary').center(_COL_WIDTH) +
+    '\n' +
+    '-'*_COL_WIDTH
+)
+
+
 class Cache(object):
     def __init__(self):
         self.key = None
@@ -323,7 +357,7 @@ class Problem(u.Canonical):
         return self._solver_stats
 
     def solve(self, *args, **kwargs):
-        """Solves the problem using the specified method.
+        """Compiles and solves the problem using the specified method.
 
         Populates the :code:`status` and :code:`value` attributes on the
         problem object as a side-effect.
@@ -333,7 +367,8 @@ class Problem(u.Canonical):
         solver : str, optional
             The solver to use. For example, 'ECOS', 'SCS', or 'OSQP'.
         verbose : bool, optional
-            Overrides the default of hiding solver output.
+            Overrides the default of hiding solver output, and prints
+            logging information describing CVXPY's compilation process.
         gp : bool, optional
             If True, parses the problem as a disciplined geometric program
             instead of a disciplined convex program.
@@ -414,7 +449,7 @@ class Problem(u.Canonical):
         """
         cls.REGISTERED_SOLVE_METHODS[name] = func
 
-    def get_problem_data(self, solver, gp=False, enforce_dpp=False):
+    def get_problem_data(self, solver, gp=False, enforce_dpp=False, verbose=False):
         """Returns the problem data used in the call to the solver.
 
         When a problem is solved, CVXPY creates a chain of reductions enclosed
@@ -488,6 +523,8 @@ class Problem(u.Canonical):
         enforce_dpp : bool, optional
             When True, a DPPError will be thrown when trying to parse a non-DPP
             problem (instead of just a warning). Defaults to False.
+        verbose : bool, optional
+            If True, print verbose output related to problem compilation.
 
         Returns
         -------
@@ -498,6 +535,7 @@ class Problem(u.Canonical):
         list
             The inverse data generated by the chain.
         """
+        start = time.time()
         key = self._cache.make_key(solver, gp)
         if key != self._cache.key:
             self._cache.invalidate()
@@ -509,8 +547,15 @@ class Problem(u.Canonical):
         else:
             solving_chain = self._cache.solving_chain
 
+        if verbose:
+            print(_COMPILATION_STR)
+
         if self._cache.param_prog is not None:
             # fast path, bypasses application of reductions
+            if verbose:
+                s.LOGGER.info(
+                        'Using cached ASA map, for faster compilation '
+                        '(bypassing reduction chain).')
             if gp:
                 dgp2dcp = self._cache.solving_chain.get(Dgp2Dcp)
                 # Parameters in the param cone prog are the logs
@@ -519,6 +564,7 @@ class Problem(u.Canonical):
                 # atoms) are unchanged.
                 old_params_to_new_params = dgp2dcp.canon_methods._parameters
                 for param in self.parameters():
+
                     if param in old_params_to_new_params:
                         old_params_to_new_params[param].value = np.log(
                             param.value)
@@ -526,15 +572,39 @@ class Problem(u.Canonical):
             data, solver_inverse_data = solving_chain.solver.apply(
                 self._cache.param_prog)
             inverse_data = self._cache.inverse_data + [solver_inverse_data]
+            self._compilation_time = time.time() - start
+            if verbose:
+                s.LOGGER.info(
+                        'Finished problem compilation '
+                        f'(took {self._compilation_time:.3e} seconds).')
         else:
-            data, inverse_data = solving_chain.apply(self)
+            if verbose:
+                solver_name = solving_chain.reductions[-1].name()
+                reduction_chain_str = ' -> '.join(
+                        type(r).__name__ for r in solving_chain.reductions)
+                s.LOGGER.info(
+                         f'Compiling problem (target solver={solver_name}).')
+                s.LOGGER.info(
+                    f'Reduction chain: '
+                    f'{reduction_chain_str}'
+                )
+            data, inverse_data = solving_chain.apply(self, verbose)
             safe_to_cache = (
                 isinstance(data, dict)
                 and s.PARAM_PROB in data
                 and not any(isinstance(reduction, EvalParams)
                             for reduction in solving_chain.reductions)
             )
+            self._compilation_time = time.time() - start
+            if verbose:
+                s.LOGGER.info(
+                        'Finished problem compilation '
+                        f'(took {self._compilation_time:.3e} seconds).')
             if safe_to_cache:
+                if verbose:
+                    s.LOGGER.info(
+                        '(Subsequent compilations of this problem, using the '
+                        'same arguments, should ' 'take less time.)')
                 self._cache.param_prog = data[s.PARAM_PROB]
                 # the last datum in inverse_data corresponds to the solver,
                 # so we shouldn't cache it
@@ -771,6 +841,9 @@ class Problem(u.Canonical):
             The optimal value for the problem, or a string indicating
             why the problem could not be solved.
         """
+        if verbose:
+            print(_HEADER)
+
         for parameter in self.parameters():
             if parameter.value is None:
                 raise error.ParameterError(
@@ -778,6 +851,31 @@ class Problem(u.Canonical):
                     "associated with it; all Parameter objects must have "
                     "values before solving a problem." % parameter.name())
 
+        if verbose:
+            n_variables = sum(np.prod(v.shape) for v in self.variables())
+            n_parameters = sum(np.prod(p.shape) for p in self.parameters())
+            s.LOGGER.info(
+                    f'Your problem has {n_variables} variables, '
+                    f'{len(self.constraints)} constraints, and '
+                    f'{n_parameters} parameters.')
+            curvatures = []
+            if self.is_dcp():
+                curvatures.append('DCP')
+            if self.is_dgp():
+                curvatures.append('DGP')
+            if self.is_dqcp():
+                curvatures.append('DQCP')
+            s.LOGGER.info(
+                    f'It is compliant with the following grammars: '
+                    f'{", ".join(curvatures)}')
+            if n_parameters == 0:
+                s.LOGGER.info(
+                    '(If you need to solve this problem multiple times, '
+                    'but with different data, consider using parameters.)')
+            s.LOGGER.info(
+                    'CVXPY will first compile your problem; then, it will '
+                    'invoke a numerical solver to obtain a solution.')
+
         if requires_grad:
             dpp_context = 'dgp' if gp else 'dcp'
             if qcp:
@@ -803,7 +901,12 @@ class Problem(u.Canonical):
             if qcp and not self.is_dcp():
                 if not self.is_dqcp():
                     raise error.DQCPError("The problem is not DQCP.")
+                if verbose:
+                    s.LOGGER.info(
+                            'Reducing DQCP problem to a one-parameter '
+                            'family of DCP problems, for bisection.')
                 reductions = [dqcp2dcp.Dqcp2Dcp()]
+                start = time.time()
                 if type(self.objective) == Maximize:
                     reductions = [FlipObjective()] + reductions
                 chain = Chain(problem=self, reductions=reductions)
@@ -813,10 +916,27 @@ class Problem(u.Canonical):
                 return self.value
 
         data, solving_chain, inverse_data = self.get_problem_data(
-            solver, gp, enforce_dpp)
+            solver, gp, enforce_dpp, verbose)
+
+        if verbose:
+            print(_NUM_SOLVER_STR)
+            s.LOGGER.info(
+                    f'Invoking solver {solving_chain.reductions[-1].name()} '
+                    'to obtain a solution.')
+        start = time.time()
         solution = solving_chain.solve_via_data(
             self, data, warm_start, verbose, kwargs)
+        end = time.time()
+        self._solve_time = end - start
         self.unpack_results(solution, solving_chain, inverse_data)
+        if verbose:
+            print(_FOOTER)
+            s.LOGGER.info(f'Problem status: {self.status}')
+            s.LOGGER.info(f'Optimal value: {self.value:.3e}')
+            s.LOGGER.info(f'Compilation took {self._compilation_time:.3e} seconds')
+            s.LOGGER.info(
+                    f'Solver (including time spent in interface) took '
+                    f'{self._solve_time:.3e} seconds')
         return self.value
 
     def backward(self):
@@ -1132,6 +1252,7 @@ class Problem(u.Canonical):
                     "Solver '%s' failed. " % chain.solver.name() +
                     "Try another solver, or solve with verbose=True for more "
                     "information.")
+
         self.unpack(solution)
         self._solver_stats = SolverStats(self._solution.attr,
                                          chain.solver.name())
